package at.jku.students;

import at.jku.ssw.Map;
import at.jku.ssw.Element;
import at.jku.ssw.DotMaker;

public class HashMapLinear extends Map {

  private Element[] table;
  
  public HashMapLinear(int size) {
    table = new Element[size];
  }

  public void put(String key, Object value) {
    int hash = hash(key);
    int address = scan(hash, key);
    if (address == -1) {
      throw new IllegalStateException("Map is full");
    }
    table[address] = new Element(key, value);
  }

  private int hash(String key) {
    return Math.abs(key.hashCode()) % table.length;
  }

  private int scan(int address, String key) {
    int address0 = address;
    while (table[address] != null 
        && (table[address].isDeleted || !table[address].key.equals(key))) {
      address = (address + 1) % table.length;
      if (address == address0) {
        return -1;
      }
    }
    return address;
  }
  
  public boolean containsKey(String key) {
    int address = scan(hash(key), key);
    return address != -1 && table[address] != null;
  }

  public Object get(String key) {
    int address = scan(hash(key), key);
    if (address != -1 && table[address] != null) {
      return table[address].value;
    } else {
      return null;
    }
  }

  public Object remove(String key) {
    int address = scan(hash(key), key);
    if (address != -1 && table[address] != null) {
      table[address].isDeleted = true;
      return table[address].value;
    } else {
      return null;
    }
  }
 
  public void rehash(int newSize) {
    Element[] old = table;
    table = new Element[newSize];
    for (int i = 0; i < old.length; i++) {
      Element e = old[i];
      if (e != null && !e.isDeleted) {
        put(e.key, e.value);
      }
    }
  }
  
  public String makeDot() {
    return DotMaker.makeDotForHashtable(table);
  }
}